@@ -40,6 +40,15 @@ struct mutex_entry
 	void *pointer;
 	unsigned count;
 };
+
+class __LOCAL rwlock_entry : public rwlock
+{
+public:
+	rwlock_entry();
+	struct rwlock_entry *next;
+	void *object;
+	unsigned count;
+};
 	
 class __LOCAL mutex_index : public mutex
 {
@@ -49,15 +58,36 @@ public:
 	mutex_index();
 };
 
+class __LOCAL rwlock_index : public mutex
+{
+public:
+	struct rwlock_entry *list;
+
+	rwlock_index();
+};
+
+static rwlock_index single_rwlock;
+static rwlock_index *rwlock_table = &single_rwlock;
 static mutex_index single_table;
 static mutex_index *mutex_table = &single_table;
 static unsigned mutex_indexing = 1;
+static unsigned rwlock_indexing = 1;
 
 mutex_index::mutex_index() : mutex()
 {
 	list = NULL;
 }
 
+rwlock_index::rwlock_index() : mutex()
+{
+	list = NULL;
+}
+
+rwlock_entry::rwlock_entry() : rwlock()
+{
+	count = 0;
+}
+
 #if !defined(_MSWINDOWS_) && !defined(__PTH__)
 Conditional::attribute Conditional::attr;
 #endif
@@ -85,7 +115,7 @@ inline pthread_t pthread_self(void)
 
 #endif
 
-static unsigned hash_address(void *ptr)
+static unsigned hash_address(void *ptr, unsigned indexing)
 {
 	assert(ptr != NULL);
 
@@ -93,7 +123,7 @@ static unsigned hash_address(void *ptr)
 	unsigned count = 0;
 	unsigned char *addr = (unsigned char *)(&ptr);
 
-	if(mutex_indexing < 2)
+	if(indexing < 2)
 		return 0;
 
 	// skip lead zeros if little endian...
@@ -105,7 +135,7 @@ static unsigned hash_address(void *ptr)
 	while(count++ < sizeof(void *) && *addr)
 		key = (key << 1) ^ *(addr++);	
 
-	return key % mutex_indexing;
+	return key % indexing;
 }
 
 ReusableAllocator::ReusableAllocator() :
@@ -838,6 +868,39 @@ void auto_protect::operator=(void *obj)
 		mutex::protect(object);
 }
 
+mutex::gaurd::gaurd()
+{
+	object = NULL;
+}
+
+mutex::gaurd::gaurd(void *obj)
+{
+	object = obj;
+	if(obj)
+		mutex::protect(object);
+}
+
+mutex::gaurd::~gaurd()
+{
+	release();
+}
+
+void mutex::gaurd::set(void *obj)
+{
+	release();
+	object = obj;
+	if(obj)
+		mutex::protect(object);
+}
+
+void mutex::gaurd::release(void)
+{
+	if(object) {
+		mutex::release(object);
+		object = NULL;
+	}
+}
+
 mutex::mutex()
 {
 #ifdef	__PTH__
@@ -860,9 +923,91 @@ void mutex::indexing(unsigned index)
 	}
 }
 
+void rwlock::indexing(unsigned index)
+{
+	if(index > 1) {
+		rwlock_table = new rwlock_index[index];
+		rwlock_indexing = index;
+	}
+}
+
+bool rwlock::shared(void *ptr, timeout_t timeout)
+{
+	rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
+	rwlock_entry *entry, *empty = NULL;
+
+	if(!ptr)
+		return false;
+
+	index->acquire();
+	entry = index->list;
+	while(entry) {
+		if(entry->count && entry->object == ptr)
+			break;
+		if(!entry->count)
+			empty = entry;
+		entry = entry->next;
+	}
+	if(!entry) {
+		if(empty)
+			entry = empty;
+		else {
+			entry = new struct rwlock_entry;
+			entry->next = index->list;
+			index->list = entry;
+		}
+	}
+	entry->object = ptr;
+	++entry->count;
+	index->release();
+	if(entry->access(timeout))
+		return true;
+	index->acquire();
+	--entry->count;
+	index->release();
+	return false;
+}			
+
+bool rwlock::exclusive(void *ptr, timeout_t timeout)
+{
+	rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
+	rwlock_entry *entry, *empty = NULL;
+
+	if(!ptr)
+		return false;
+
+	index->acquire();
+	entry = index->list;
+	while(entry) {
+		if(entry->count && entry->object == ptr)
+			break;
+		if(!entry->count)
+			empty = entry;
+		entry = entry->next;
+	}
+	if(!entry) {
+		if(empty)
+			entry = empty;
+		else {
+			entry = new struct rwlock_entry;
+			entry->next = index->list;
+			index->list = entry;
+		}
+	}
+	entry->object = ptr;
+	++entry->count;
+	index->release();
+	if(entry->modify(timeout))
+		return true;
+	index->acquire();
+	--entry->count;
+	index->release();
+	return false;
+}			
+
 void mutex::protect(void *ptr)
 {
-	mutex_index *index = &mutex_table[hash_address(ptr)];
+	mutex_index *index = &mutex_table[hash_address(ptr, mutex_indexing)];
 	mutex_entry *entry, *empty = NULL;
 
 	if(!ptr)
@@ -895,9 +1040,33 @@ void mutex::protect(void *ptr)
 	pthread_mutex_lock(&entry->mutex);
 }	
 
+void rwlock::release(void *ptr)
+{
+	rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
+	rwlock_entry *entry;
+
+	if(!ptr)
+		return;
+
+	index->acquire();
+	entry = index->list;
+	while(entry) {
+		if(entry->count && entry->object == ptr)
+			break;
+		entry = entry->next;
+	}
+
+	assert(entry);
+	if(entry) {
+		entry->release();
+		--entry->count;
+	}
+	index->release();
+}	
+
 void mutex::release(void *ptr)
 {
-	mutex_index *index = &mutex_table[hash_address(ptr)];
+	mutex_index *index = &mutex_table[hash_address(ptr, mutex_indexing)];
 	mutex_entry *entry;
 
 	if(!ptr)
@@ -1103,7 +1272,6 @@ bool TimedEvent::sync(void)
 bool TimedEvent::wait(timeout_t timeout) 
 {
 	bool result = true;
-	struct timespec ts;
 
 	pthread_mutex_lock(&mutex);
 	operator+=(timeout);
@@ -1458,7 +1626,7 @@ void Thread::setPriority(void)
 	int policy;
 	struct sched_param sp;
 	pthread_t ptid = pthread_self();
-	int pri;
+	int pri = 0;
 
 	if(!priority)
 		return;
